@@ -41,6 +41,7 @@ class CoverDownloader(object):
         '.png': lambda d: d.startswith(b'\x89PNG\r\n\x1a\n\x00'),
         '.jpg': lambda d: d.startswith(b'\xff\xd8'),
         '.gif': lambda d: d.startswith(b'GIF89a') or d.startswith(b'GIF87a'),
+        '.ico': lambda d: d.startswith(b'\0\0\1\0'),
     }
 
     EXTENSIONS = list(SUPPORTED_EXTENSIONS.keys())
@@ -84,9 +85,13 @@ class CoverDownloader(object):
 
             try:
                 logger.info('Downloading cover art: %s', cover_url)
-                data = util.urlopen(cover_url, timeout=self.TIMEOUT).read()
+                response = util.urlopen(cover_url, timeout=self.TIMEOUT)
+                if response.status_code != 200:
+                    msg = '%s returned status code %d' % (cover_url, response.status_code)
+                    raise ValueError(msg)
+                data = response.content
             except Exception as e:
-                logger.warn('Cover art download failed: %s', e)
+                logger.warning('Cover art download failed: %s', e)
                 return self._fallback_filename(title)
 
             try:
@@ -108,7 +113,7 @@ class CoverDownloader(object):
 
                 return filename + extension
             except Exception as e:
-                logger.warn('Cannot save cover art', exc_info=True)
+                logger.warning('Cannot save cover art', exc_info=True)
 
         # Fallback to cover art based on the podcast title
         return self._fallback_filename(title)
