@@ -89,7 +89,7 @@ def directory_entry_from_opml(url):
 
 def directory_entry_from_mygpo_json(url):
     return [DirectoryEntry(d['title'], d['url'], d['logo_url'], d['subscribers'], d['description'])
-            for d in json.load(util.urlopen(url))]
+            for d in util.urlopen(url).json()]
 
 
 class GPodderNetSearchProvider(Provider):
@@ -152,7 +152,7 @@ class GPodderNetTagsProvider(Provider):
         return directory_entry_from_mygpo_json('http://gpodder.net/api/2/tag/%s/50.json' % urllib.parse.quote(tag))
 
     def get_tags(self):
-        return [DirectoryTag(d['tag'], d['usage']) for d in json.load(util.urlopen('http://gpodder.net/api/2/tags/40.json'))]
+        return [DirectoryTag(d['tag'], d['usage']) for d in util.urlopen('http://gpodder.net/api/2/tags/40.json').json()]
 
 
 class SoundcloudSearchProvider(Provider):
