@@ -27,6 +27,7 @@
 
 import collections
 import email
+import glob
 import logging
 import mimetypes
 import os
@@ -188,6 +189,9 @@ class ContentRange(object):
 class DownloadCancelledException(Exception): pass
 
 
+class DownloadNoURLException(Exception): pass
+
+
 class gPodderDownloadHTTPError(Exception):
     def __init__(self, url, error_code, error_message):
         self.url = url
@@ -227,7 +231,7 @@ class DownloadURLOpener:
 # The following is based on Python's urllib.py "URLopener.retrieve"
 # Also based on http://mail.python.org/pipermail/python-list/2001-October/110069.html
 
-    def retrieve_resume(self, url, filename, reporthook=None, data=None):
+    def retrieve_resume(self, url, filename, reporthook=None, data=None, disable_auth=False):
         """Download files from an URL; return (headers, real_url)
 
         Resumes a download if the local filename exists and
@@ -240,7 +244,7 @@ class DownloadURLOpener:
             'User-agent': gpodder.user_agent
         }
 
-        if self.channel.auth_username or self.channel.auth_password:
+        if (self.channel.auth_username or self.channel.auth_password) and not disable_auth:
             logger.debug('Authenticating as "%s"', self.channel.auth_username)
             auth = (self.channel.auth_username, self.channel.auth_password)
         else:
@@ -254,7 +258,7 @@ class DownloadURLOpener:
                 if current_size > 0:
                     headers['Range'] = 'bytes=%s-' % (current_size)
             except:
-                logger.warn('Cannot resume download: %s', filename, exc_info=True)
+                logger.warning('Cannot resume download: %s', filename, exc_info=True)
                 tfp = None
                 current_size = 0
 
@@ -273,7 +277,11 @@ class DownloadURLOpener:
             try:
                 resp.raise_for_status()
             except HTTPError as e:
-                raise gPodderDownloadHTTPError(url, resp.status_code, str(e))
+                if auth is not None:
+                    # Try again without authentication (bug 1296)
+                    return self.retrieve_resume(url, filename, reporthook, data, True)
+                else:
+                    raise gPodderDownloadHTTPError(url, resp.status_code, str(e))
 
             headers = resp.headers
 
@@ -288,7 +296,7 @@ class DownloadURLOpener:
                     tfp.close()
                     tfp = open(filename, 'wb')
                     current_size = 0
-                    logger.warn('Cannot resume: Invalid Content-Range (RFC2616).')
+                    logger.warning('Cannot resume: Invalid Content-Range (RFC2616).')
 
             result = headers, resp.url
             bs = 1024 * 8
@@ -380,14 +388,14 @@ class DownloadQueueWorker(object):
             if not self.continue_check_callback(self):
                 return
 
-            try:
-                task = self.queue.get_next()
-                logger.info('%s is processing: %s', self, task)
-                task.run()
-                task.recycle()
-            except StopIteration as e:
+            task = self.queue.get_next()
+            if not task:
                 logger.info('No more tasks for %s to carry out.', self)
                 break
+            logger.info('%s is processing: %s', self, task)
+            task.run()
+            task.recycle()
+
         self.exit_callback(self)
 
 
@@ -402,6 +410,7 @@ class ForceDownloadWorker(object):
         logger.info('Starting new thread: %s', self)
         logger.info('%s is processing: %s', self, self.task)
         self.task.run()
+        self.task.recycle()
 
 
 class DownloadQueueManager(object):
@@ -432,12 +441,12 @@ class DownloadQueueManager(object):
             work_count = self.tasks.available_work_count()
             if self._config.max_downloads_enabled:
                 # always allow at least 1 download
-                max_downloads = max(int(self._config.max_downloads), 1)
-                spawn_limit = max_downloads - len(self.worker_threads)
+                spawn_limit = max(int(self._config.max_downloads), 1)
             else:
                 spawn_limit = self._config.limit.downloads.concurrent_max
-            logger.info('%r tasks to do, can start at most %r threads', work_count, spawn_limit)
-            for i in range(0, min(work_count, spawn_limit)):
+            running = len(self.worker_threads)
+            logger.info('%r tasks to do, can start at most %r threads, %r threads currently running', work_count, spawn_limit, running)
+            for i in range(0, min(work_count, spawn_limit - running)):
                 # We have to create a new thread here, there's work to do
                 logger.info('Starting new worker thread.')
 
@@ -450,16 +459,21 @@ class DownloadQueueManager(object):
         self.__spawn_threads()
 
     def force_start_task(self, task):
-        if self.tasks.set_downloading(task):
-            worker = ForceDownloadWorker(task)
-            util.run_in_background(worker.run)
+        with task:
+            if task.status in (task.QUEUED, task.PAUSED, task.CANCELLED, task.FAILED):
+                task.status = task.DOWNLOADING
+                worker = ForceDownloadWorker(task)
+                util.run_in_background(worker.run)
 
     def queue_task(self, task):
         """Marks a task as queued
         """
-        task.status = DownloadTask.QUEUED
+        self.tasks.queue_task(task)
         self.__spawn_threads()
 
+    def has_workers(self):
+        return len(self.worker_threads) > 0
+
 
 class DownloadTask(object):
     """An object representing the download task of an episode
@@ -467,7 +481,7 @@ class DownloadTask(object):
     You can create a new download task like this:
 
         task = DownloadTask(episode, gpodder.config.Config(CONFIGFILE))
-        task.status = DownloadTask.DOWNLOADING
+        task.status = DownloadTask.QUEUED
         task.run()
 
     While the download is in progress, you can access its properties:
@@ -484,7 +498,8 @@ class DownloadTask(object):
 
     You can cancel a running download task by setting its status:
 
-        task.status = DownloadTask.CANCELLED
+        with task:
+            task.status = DownloadTask.CANCELLING
 
     The task will then abort as soon as possible (due to the nature
     of downloading data, this can take a while when the Internet is
@@ -535,9 +550,9 @@ class DownloadTask(object):
     The same thing works for failed downloads ("notify_as_failed()").
     """
     # Possible states this download task can be in
-    STATUS_MESSAGE = (_('Added'), _('Queued'), _('Downloading'),
-            _('Finished'), _('Failed'), _('Cancelled'), _('Paused'))
-    (INIT, QUEUED, DOWNLOADING, DONE, FAILED, CANCELLED, PAUSED) = list(range(7))
+    STATUS_MESSAGE = (_('Queued'), _('Queued'), _('Downloading'),
+            _('Finished'), _('Failed'), _('Cancelling'), _('Cancelled'), _('Pausing'), _('Paused'))
+    (NEW, QUEUED, DOWNLOADING, DONE, FAILED, CANCELLING, CANCELLED, PAUSING, PAUSED) = list(range(9))
 
     # Wheter this task represents a file download or a device sync operation
     ACTIVITY_DOWNLOAD, ACTIVITY_SYNCHRONIZE = list(range(2))
@@ -548,6 +563,12 @@ class DownloadTask(object):
     def __str__(self):
         return self.__episode.title
 
+    def __enter__(self):
+        return self.__lock.acquire()
+
+    def __exit__(self, type, value, traceback):
+        self.__lock.release()
+
     def __get_status(self):
         return self.__status
 
@@ -599,17 +620,61 @@ class DownloadTask(object):
 
     downloader = property(fget=__get_downloader, fset=__set_downloader)
 
+    def can_queue(self):
+        return self.status in (self.CANCELLED, self.PAUSED, self.FAILED)
+
+    def unpause(self):
+        with self:
+            # Resume a downloading task that was transitioning to paused
+            if self.status == self.PAUSING:
+                self.status = self.DOWNLOADING
+
+    def can_pause(self):
+        return self.status in (self.DOWNLOADING, self.QUEUED)
+
+    def pause(self):
+        with self:
+            # Pause a queued download
+            if self.status == self.QUEUED:
+                self.status = self.PAUSED
+            # Request pause of a running download
+            elif self.status == self.DOWNLOADING:
+                self.status = self.PAUSING
+                # download rate limited tasks sleep and take longer to transition from the PAUSING state to the PAUSED state
+
+    def can_cancel(self):
+        return self.status in (self.DOWNLOADING, self.QUEUED, self.PAUSED, self.FAILED)
+
     def cancel(self):
-        if self.status in (self.DOWNLOADING, self.QUEUED):
-            self.status = self.CANCELLED
+        with self:
+            # Cancelling directly is allowed if the task isn't currently downloading
+            if self.status in (self.QUEUED, self.PAUSED, self.FAILED):
+                self.status = self.CANCELLING
+                # Call run, so the partial file gets deleted, and task recycled
+                self.run()
+            # Otherwise request cancellation
+            elif self.status == self.DOWNLOADING:
+                self.status = self.CANCELLING
+
+    def can_remove(self):
+        return self.status in (self.CANCELLED, self.FAILED, self.DONE)
+
+    def delete_partial_files(self):
+        temporary_files = [self.tempname]
+        # youtube-dl creates .partial.* files for adaptive formats
+        temporary_files += glob.glob('%s.*' % self.tempname)
+
+        for tempfile in temporary_files:
+            util.delete_file(tempfile)
 
     def removed_from_list(self):
         if self.status != self.DONE:
-            util.delete_file(self.tempname)
+            self.delete_partial_files()
 
     def __init__(self, episode, config, downloader=None):
         assert episode.download_task is None
-        self.__status = DownloadTask.INIT
+        self.__lock = threading.RLock()
+        self.__status = DownloadTask.NEW
         self.__activity = DownloadTask.ACTIVITY_DOWNLOAD
         self.__status_changed = True
         self.__episode = episode
@@ -655,6 +720,11 @@ class DownloadTask(object):
         # Store a reference to this task in the episode
         episode.download_task = self
 
+    def reuse(self):
+        if not os.path.exists(self.tempname):
+            # partial file was deleted when cancelled, recreate it
+            open(self.tempname, 'w').close()
+
     def notify_as_finished(self):
         if self.status == DownloadTask.DONE:
             if self._notification_shown:
@@ -699,10 +769,10 @@ class DownloadTask(object):
 
         self.calculate_speed(count, blockSize)
 
-        if self.status == DownloadTask.CANCELLED:
+        if self.status == DownloadTask.CANCELLING:
             raise DownloadCancelledException()
 
-        if self.status == DownloadTask.PAUSED:
+        if self.status == DownloadTask.PAUSING:
             raise DownloadCancelledException()
 
     def calculate_speed(self, count, blockSize):
@@ -746,37 +816,51 @@ class DownloadTask(object):
                     time.sleep(delay)
 
     def recycle(self):
-        self.episode.download_task = None
+        if self.status not in (self.FAILED, self.PAUSED):
+            self.episode.download_task = None
+
+    def set_episode_download_task(self):
+        if not self.episode.download_task:
+            self.episode.download_task = self
 
     def run(self):
         # Speed calculation (re-)starts here
         self.__start_time = 0
         self.__start_blocks = 0
 
-        # If the download has already been cancelled, skip it
-        if self.status == DownloadTask.CANCELLED:
-            util.delete_file(self.tempname)
-            self.progress = 0.0
-            self.speed = 0.0
-            self.recycle()
-            return False
+        # If the download has already been cancelled/paused, skip it
+        with self:
+            if self.status == DownloadTask.CANCELLING:
+                self.status = DownloadTask.CANCELLED
+                self.__episode._download_error = None
+                self.delete_partial_files()
+                self.progress = 0.0
+                self.speed = 0.0
+                self.recycle()
+                return False
 
-        # We only start this download if its status is "downloading"
-        if self.status != DownloadTask.DOWNLOADING:
-            return False
+            if self.status == DownloadTask.PAUSING:
+                self.status = DownloadTask.PAUSED
+                return False
 
-        # We are downloading this file right now
-        self.status = DownloadTask.DOWNLOADING
-        self._notification_shown = False
+            # We only start this download if its status is downloading
+            if self.status != DownloadTask.DOWNLOADING:
+                return False
 
-        # Restore a reference to this task in the episode
-        # when running a recycled task following a pause or failed
-        # see #649
-        if not self.episode.download_task:
-            self.episode.download_task = self
+            # We are downloading this file right now
+            self._notification_shown = False
+
+            # Restore a reference to this task in the episode
+            # when running a recycled task following a pause or failed
+            # see #649
+            self.set_episode_download_task()
 
         url = self.__episode.url
+        result = DownloadTask.DOWNLOADING
         try:
+            if url == '':
+                raise DownloadNoURLException()
+
             if self.downloader:
                 downloader = self.downloader.custom_downloader(self._config, self.episode)
             else:
@@ -849,16 +933,21 @@ class DownloadTask(object):
             self.__episode.on_downloaded(self.filename)
         except DownloadCancelledException:
             logger.info('Download has been cancelled/paused: %s', self)
-            if self.status == DownloadTask.CANCELLED:
-                util.delete_file(self.tempname)
+            if self.status == DownloadTask.CANCELLING:
+                self.__episode._download_error = None
+                self.delete_partial_files()
                 self.progress = 0.0
                 self.speed = 0.0
+            result = DownloadTask.CANCELLED
+        except DownloadNoURLException:
+            result = DownloadTask.FAILED
+            self.error_message = _('Episode has no URL to download')
         except urllib.error.ContentTooShortError as ctse:
-            self.status = DownloadTask.FAILED
+            result = DownloadTask.FAILED
             self.error_message = _('Missing content from server')
         except ConnectionError as ce:
             # special case request exception
-            self.status = DownloadTask.FAILED
+            result = DownloadTask.FAILED
             logger.error('Download failed: %s', str(ce), exc_info=True)
             d = {'host': ce.args[0].pool.host, 'port': ce.args[0].pool.port}
             self.error_message = _("Couldn't connect to server %(host)s:%(port)s" % d)
@@ -868,38 +957,50 @@ class DownloadTask(object):
                 re = re.args[0]
             logger.error('%s while downloading "%s"', str(re),
                     self.__episode.title, exc_info=True)
-            self.status = DownloadTask.FAILED
+            result = DownloadTask.FAILED
             d = {'error': str(re)}
             self.error_message = _('Request Error: %(error)s') % d
         except IOError as ioe:
             logger.error('%s while downloading "%s": %s', ioe.strerror,
                     self.__episode.title, ioe.filename, exc_info=True)
-            self.status = DownloadTask.FAILED
+            result = DownloadTask.FAILED
             d = {'error': ioe.strerror, 'filename': ioe.filename}
             self.error_message = _('I/O Error: %(error)s: %(filename)s') % d
         except gPodderDownloadHTTPError as gdhe:
             logger.error('HTTP %s while downloading "%s": %s',
                     gdhe.error_code, self.__episode.title, gdhe.error_message,
                     exc_info=True)
-            self.status = DownloadTask.FAILED
+            result = DownloadTask.FAILED
             d = {'code': gdhe.error_code, 'message': gdhe.error_message}
             self.error_message = _('HTTP Error %(code)s: %(message)s') % d
         except Exception as e:
-            self.status = DownloadTask.FAILED
+            result = DownloadTask.FAILED
             logger.error('Download failed: %s', str(e), exc_info=True)
             self.error_message = _('Error: %s') % (str(e),)
 
-        if self.status == DownloadTask.DOWNLOADING:
-            # Everything went well - we're done
-            self.status = DownloadTask.DONE
-            if self.total_size <= 0:
-                self.total_size = util.calculate_size(self.filename)
-                logger.info('Total size updated to %d', self.total_size)
-            self.progress = 1.0
-            gpodder.user_extensions.on_episode_downloaded(self.__episode)
-            return True
+        with self:
+            if result == DownloadTask.DOWNLOADING:
+                # Everything went well - we're done (even if the task was cancelled/paused,
+                # since it's finished we might as well mark it done)
+                self.status = DownloadTask.DONE
+                if self.total_size <= 0:
+                    self.total_size = util.calculate_size(self.filename)
+                    logger.info('Total size updated to %d', self.total_size)
+                self.progress = 1.0
+                gpodder.user_extensions.on_episode_downloaded(self.__episode)
+                return True
 
-        self.speed = 0.0
+            self.speed = 0.0
+
+            if result == DownloadTask.FAILED:
+                self.status = DownloadTask.FAILED
+                self.__episode._download_error = self.error_message
+
+            # cancelled/paused -- update state to mark it as safe to manipulate this task again
+            elif self.status == DownloadTask.PAUSING:
+                self.status = DownloadTask.PAUSED
+            elif self.status == DownloadTask.CANCELLING:
+                self.status = DownloadTask.CANCELLED
 
         # We finished, but not successfully (at least not really)
         return False
