@@ -26,7 +26,8 @@ from gi.repository import Gdk, Gtk, Pango
 import gpodder
 from gpodder import util, vimeo, youtube
 from gpodder.gtkui.desktopfile import PlayerListModel
-from gpodder.gtkui.interface.common import BuilderWidget, TreeViewHelper
+from gpodder.gtkui.interface.common import (BuilderWidget, TreeViewHelper,
+                                            show_message_dialog)
 from gpodder.gtkui.interface.configeditor import gPodderConfigEditor
 
 logger = logging.getLogger(__name__)
@@ -184,6 +185,7 @@ class gPodderPreferences(BuilderWidget):
     C_TOGGLE, C_LABEL, C_EXTENSION, C_SHOW_TOGGLE = list(range(4))
 
     def new(self):
+        self.gPodderPreferences.set_transient_for(self.parent_widget)
         for cb in (self.combo_audio_player_app, self.combo_video_player_app):
             cellrenderer = Gtk.CellRendererPixbuf()
             cb.pack_start(cellrenderer, False)
@@ -207,6 +209,7 @@ class gPodderPreferences(BuilderWidget):
         self.preferred_youtube_format_model = YouTubeVideoFormatListModel(self._config)
         self.combobox_preferred_youtube_format.set_model(self.preferred_youtube_format_model)
         cellrenderer = Gtk.CellRendererText()
+        cellrenderer.set_property('ellipsize', Pango.EllipsizeMode.END)
         self.combobox_preferred_youtube_format.pack_start(cellrenderer, True)
         self.combobox_preferred_youtube_format.add_attribute(cellrenderer, 'text', self.preferred_youtube_format_model.C_CAPTION)
         self.combobox_preferred_youtube_format.set_active(self.preferred_youtube_format_model.get_index())
@@ -214,6 +217,7 @@ class gPodderPreferences(BuilderWidget):
         self.preferred_youtube_hls_format_model = YouTubeVideoHLSFormatListModel(self._config)
         self.combobox_preferred_youtube_hls_format.set_model(self.preferred_youtube_hls_format_model)
         cellrenderer = Gtk.CellRendererText()
+        cellrenderer.set_property('ellipsize', Pango.EllipsizeMode.END)
         self.combobox_preferred_youtube_hls_format.pack_start(cellrenderer, True)
         self.combobox_preferred_youtube_hls_format.add_attribute(cellrenderer, 'text', self.preferred_youtube_hls_format_model.C_CAPTION)
         self.combobox_preferred_youtube_hls_format.set_active(self.preferred_youtube_hls_format_model.get_index())
@@ -221,6 +225,7 @@ class gPodderPreferences(BuilderWidget):
         self.preferred_vimeo_format_model = VimeoVideoFormatListModel(self._config)
         self.combobox_preferred_vimeo_format.set_model(self.preferred_vimeo_format_model)
         cellrenderer = Gtk.CellRendererText()
+        cellrenderer.set_property('ellipsize', Pango.EllipsizeMode.END)
         self.combobox_preferred_vimeo_format.pack_start(cellrenderer, True)
         self.combobox_preferred_vimeo_format.add_attribute(cellrenderer, 'text', self.preferred_vimeo_format_model.C_CAPTION)
         self.combobox_preferred_vimeo_format.set_active(self.preferred_vimeo_format_model.get_index())
@@ -254,6 +259,9 @@ class gPodderPreferences(BuilderWidget):
         self.combo_auto_download.add_attribute(cellrenderer, 'text', NewEpisodeActionList.C_CAPTION)
         self.combo_auto_download.set_active(self.auto_download_model.get_index())
 
+        self._config.connect_gtk_togglebutton('check_connection',
+                                              self.checkbutton_check_connection)
+
         if self._config.auto_remove_played_episodes:
             adjustment_expiration = self.hscale_expiration.get_adjustment()
             if self._config.episode_old_age > adjustment_expiration.get_upper():
@@ -290,6 +298,8 @@ class gPodderPreferences(BuilderWidget):
                                               self.checkbutton_create_playlists)
         self._config.connect_gtk_togglebutton('device_sync.playlists.two_way_sync',
                                               self.checkbutton_delete_using_playlists)
+        self._config.connect_gtk_togglebutton('device_sync.delete_deleted_episodes',
+                                              self.checkbutton_delete_deleted_episodes)
 
         # Have to do this before calling set_active on checkbutton_enable
         self._enable_mygpo = self._config.mygpo.enabled
@@ -311,10 +321,35 @@ class gPodderPreferences(BuilderWidget):
 
         gpodder.user_extensions.on_ui_object_available('preferences-gtk', self)
 
+        self.inject_extensions_preferences(init=True)
+
+        self.prefs_stack.foreach(self._wrap_checkbox_labels)
+
+    def _wrap_checkbox_labels(self, w, *args):
+        if w.get_name().startswith("no_label_wrap"):
+            return
+        elif isinstance(w, Gtk.CheckButton):
+            label = w.get_child()
+            label.set_line_wrap(True)
+        elif isinstance(w, Gtk.Container):
+            w.foreach(self._wrap_checkbox_labels)
+
+    def inject_extensions_preferences(self, init=False):
+        if not init:
+            # remove preferences buttons for all extensions
+            for child in self.prefs_stack.get_children():
+                if child.get_name().startswith("extension."):
+                    self.prefs_stack.remove(child)
+
+        # add preferences buttons for all extensions
         result = gpodder.user_extensions.on_preferences()
         if result:
             for label, callback in result:
-                self.notebook.append_page(callback(), Gtk.Label(label))
+                page = callback()
+                name = "extension." + label
+                page.set_name(name)
+                page.foreach(self._wrap_checkbox_labels)
+                self.prefs_stack.add_titled(page, name, label)
 
     def _extensions_select_function(self, selection, model, path, path_currently_selected):
         return model.get_value(model.get_iter(path), self.C_SHOW_TOGGLE)
@@ -443,6 +478,7 @@ class gPodderPreferences(BuilderWidget):
                 self.on_extension_enabled(container.module)
             else:
                 self.on_extension_disabled(container.module)
+            self.inject_extensions_preferences()
         elif container.error is not None:
             if hasattr(container.error, 'message'):
                 error_msg = container.error.message
@@ -456,13 +492,11 @@ class gPodderPreferences(BuilderWidget):
         if not container or not model:
             return
 
-        # This is one ugly hack, but it displays the attributes of
-        # the metadata object of the container..
-        info = '\n'.join('<b>%s:</b> %s' %
-                         tuple(map(html.escape, list(map(str, (key, value)))))
-                         for key, value in container.metadata.get_sorted())
+        info = '\n'.join('<b>{}:</b> {}'.format(html.escape(key), html.escape(value))
+                         for key, value in container.metadata.get_sorted()
+                         if key not in ('title', 'description'))
 
-        self.show_message(info, _('Extension module info'), important=True)
+        self.show_message_details(container.metadata.title, container.metadata.description, info)
 
     def open_weblink(self, w, url):
         util.open_website(url)
@@ -618,7 +652,8 @@ class gPodderPreferences(BuilderWidget):
             children = self.btn_playlistfolder.get_children()
             if children:
                 label = children.pop()
-                label.set_alignment(0., .5)
+                label.set_ellipsize(Pango.EllipsizeMode.START)
+                label.set_xalign(0.0)
         else:
             self.btn_playlistfolder.set_sensitive(False)
             self.btn_playlistfolder.set_label('')
@@ -637,16 +672,13 @@ class gPodderPreferences(BuilderWidget):
             self.combobox_on_sync.set_sensitive(False)
             self.checkbutton_skip_played_episodes.set_sensitive(False)
         elif device_type == 'filesystem':
-            self.btn_filesystemMountpoint.set_label(self._config.device_sync.device_folder)
+            self.btn_filesystemMountpoint.set_label(self._config.device_sync.device_folder or "")
             self.btn_filesystemMountpoint.set_sensitive(True)
             self.checkbutton_create_playlists.set_sensitive(True)
-            children = self.btn_filesystemMountpoint.get_children()
-            if children:
-                label = children.pop()
-                label.set_alignment(0., .5)
             self.toggle_playlist_interface(self._config.device_sync.playlists.create)
             self.combobox_on_sync.set_sensitive(True)
             self.checkbutton_skip_played_episodes.set_sensitive(True)
+            self.checkbutton_delete_deleted_episodes.set_sensitive(True)
         elif device_type == 'ipod':
             self.btn_filesystemMountpoint.set_label(self._config.device_sync.device_folder)
             self.btn_filesystemMountpoint.set_sensitive(True)
@@ -654,29 +686,28 @@ class gPodderPreferences(BuilderWidget):
             self.toggle_playlist_interface(False)
             self.checkbutton_delete_using_playlists.set_sensitive(False)
             self.combobox_on_sync.set_sensitive(False)
-            self.checkbutton_skip_played_episodes.set_sensitive(False)
+            self.checkbutton_skip_played_episodes.set_sensitive(True)
+            self.checkbutton_delete_deleted_episodes.set_sensitive(True)
 
-            children = self.btn_filesystemMountpoint.get_children()
-            if children:
-                label = children.pop()
-                label.set_alignment(0., .5)
-
-        else:
-            # TODO: Add support for iPod and MTP devices
-            pass
+        children = self.btn_filesystemMountpoint.get_children()
+        if children:
+            label = children.pop()
+            label.set_ellipsize(Pango.EllipsizeMode.START)
+            label.set_xalign(0.0)
 
     def on_btn_device_mountpoint_clicked(self, widget):
         fs = Gtk.FileChooserDialog(title=_('Select folder for mount point'),
                 action=Gtk.FileChooserAction.SELECT_FOLDER)
-        fs.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        fs.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
-        fs.set_current_folder(self.btn_filesystemMountpoint.get_label())
+        fs.set_local_only(False)
+        fs.add_button(_('_Cancel'), Gtk.ResponseType.CANCEL)
+        fs.add_button(_('_Open'), Gtk.ResponseType.OK)
+
+        fs.set_uri(self.btn_filesystemMountpoint.get_label() or "")
         if fs.run() == Gtk.ResponseType.OK:
-            filename = fs.get_filename()
             if self._config.device_sync.device_type == 'filesystem':
-                self._config.device_sync.device_folder = filename
+                self._config.device_sync.device_folder = fs.get_uri()
             elif self._config.device_sync.device_type == 'ipod':
-                self._config.device_sync.device_folder = filename
+                self._config.device_sync.device_folder = fs.get_filename()
             # Request an update of the mountpoint button
             self.on_combobox_device_type_changed(None)
 
@@ -685,18 +716,29 @@ class gPodderPreferences(BuilderWidget):
     def on_btn_playlist_folder_clicked(self, widget):
         fs = Gtk.FileChooserDialog(title=_('Select folder for playlists'),
                 action=Gtk.FileChooserAction.SELECT_FOLDER)
-        fs.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        fs.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
-        fs.set_current_folder(self.btn_playlistfolder.get_label())
-        if fs.run() == Gtk.ResponseType.OK:
-            filename = util.relpath(self._config.device_sync.device_folder,
-                                    fs.get_filename())
+        fs.set_local_only(False)
+        fs.add_button(_('_Cancel'), Gtk.ResponseType.CANCEL)
+        fs.add_button(_('_Open'), Gtk.ResponseType.OK)
+
+        device_folder = util.new_gio_file(self._config.device_sync.device_folder)
+        playlists_folder = device_folder.resolve_relative_path(self._config.device_sync.playlists.folder)
+        fs.set_file(playlists_folder)
+
+        while fs.run() == Gtk.ResponseType.OK:
+            filename = util.relpath(fs.get_uri(),
+                                    self._config.device_sync.device_folder)
+            if not filename:
+                show_message_dialog(fs, _('The playlists folder must be on the device'))
+                continue
+
             if self._config.device_sync.device_type == 'filesystem':
                 self._config.device_sync.playlists.folder = filename
-                self.btn_playlistfolder.set_label(filename)
+                self.btn_playlistfolder.set_label(filename or "")
                 children = self.btn_playlistfolder.get_children()
                 if children:
                     label = children.pop()
-                    label.set_alignment(0., .5)
+                    label.set_ellipsize(Pango.EllipsizeMode.START)
+                    label.set_xalign(0.0)
+            break
 
         fs.destroy()
