@@ -28,6 +28,7 @@ import collections
 import datetime
 import glob
 import hashlib
+import json
 import logging
 import os
 import re
@@ -110,13 +111,13 @@ class PodcastParserFeed(Feed):
     def get_link(self):
         vid = youtube.get_youtube_id(self.feed['url'])
         if vid is not None:
-            self.feed['link'] = youtube.get_channel_id_url(self.feed['url'])
+            self.feed['link'] = youtube.get_channel_id_url(self.feed['url'], self.fetcher.feed_data)
         return self.feed.get('link')
 
     def get_description(self):
         vid = youtube.get_youtube_id(self.feed['url'])
         if vid is not None:
-            self.feed['description'] = youtube.get_channel_desc(self.feed['url'])
+            self.feed['description'] = youtube.get_channel_desc(self.feed['url'], self.fetcher.feed_data)
         return self.feed.get('description')
 
     def get_cover_url(self):
@@ -161,7 +162,7 @@ class PodcastParserFeed(Feed):
                 num_duplicate_guids += 1
                 channel._update_error = ('Discarded {} episode(s) with non-unique GUID, contact the podcast publisher to fix this issue.'
                         .format(num_duplicate_guids))
-                logger.warn('Discarded episode with non-unique GUID, contact the podcast publisher to fix this issue. [%s] [%s]',
+                logger.warning('Discarded episode with non-unique GUID, contact the podcast publisher to fix this issue. [%s] [%s]',
                         channel.title, episode.title)
                 continue
 
@@ -169,10 +170,20 @@ class PodcastParserFeed(Feed):
             # Detect (and update) existing episode based on GUIDs
             existing_episode = existing_guids.get(episode.guid, None)
             if existing_episode:
+                if existing_episode.total_time == 0 and 'youtube' in episode.url:
+                    # query duration for existing youtube episodes that haven't been downloaded or queried
+                    # such as live streams after they have ended
+                    existing_episode.total_time = youtube.get_total_time(episode)
+
                 existing_episode.update_from(episode)
+                existing_episode.cache_text_description()
                 existing_episode.save()
                 continue
+            elif episode.total_time == 0 and 'youtube' in episode.url:
+                # query duration for new youtube episodes
+                episode.total_time = youtube.get_total_time(episode)
 
+            episode.cache_text_description()
             episode.save()
             new_episodes.append(episode)
         return new_episodes, seen_guids
@@ -207,7 +218,8 @@ class gPodderFetcher(feedcore.Fetcher):
         url = vimeo.get_real_channel_url(url)
         return url
 
-    def parse_feed(self, url, data_stream, headers, status, max_episodes=0, **kwargs):
+    def parse_feed(self, url, feed_data, data_stream, headers, status, max_episodes=0, **kwargs):
+        self.feed_data = feed_data
         try:
             feed = podcastparser.parse(url, data_stream)
             feed['url'] = url
@@ -262,7 +274,7 @@ class PodcastEpisode(PodcastModelObject):
     MAX_FILENAME_LENGTH = 120  # without extension
     MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial.webm")  # with extension
 
-    __slots__ = schema.EpisodeColumns
+    __slots__ = schema.EpisodeColumns + ('_download_error', '_text_description',)
 
     def _deprecated(self):
         raise Exception('Property is deprecated!')
@@ -280,28 +292,43 @@ class PodcastEpisode(PodcastModelObject):
         episode.guid = entry['guid']
         episode.title = entry['title']
         episode.link = entry['link']
-        episode.description = entry['description']
+        episode.episode_art_url = entry.get('episode_art_url')
         if entry.get('description_html'):
+            episode.description = ''
             episode.description_html = entry['description_html']
+        else:
+            episode.description = util.remove_html_tags(entry['description'] or '')
+            episode.description_html = ''
 
         episode.total_time = entry['total_time']
         episode.published = entry['published']
         episode.payment_url = entry['payment_url']
+        episode.chapters = None
+        if entry.get("chapters"):
+            episode.chapters = json.dumps(entry["chapters"])
 
         audio_available = any(enclosure['mime_type'].startswith('audio/') for enclosure in entry['enclosures'])
         video_available = any(enclosure['mime_type'].startswith('video/') for enclosure in entry['enclosures'])
+        link_has_media = False
+        if not (audio_available or video_available):
+            _url = episode.url
+            episode.url = util.normalize_feed_url(entry['link'])
+            # Check if any extensions (e.g. youtube-dl) support the link
+            link_has_media = registry.custom_downloader.resolve(None, None, episode) is not None
+            episode.url = _url
+        media_available = audio_available or video_available or link_has_media
 
         for enclosure in entry['enclosures']:
             episode.mime_type = enclosure['mime_type']
 
             # Skip images in feeds if audio or video is available (bug 979)
             # This must (and does) also look in Media RSS enclosures (bug 1430)
-            if episode.mime_type.startswith('image/') and (audio_available or video_available):
+            if episode.mime_type.startswith('image/') and media_available:
                 continue
 
             # If we have audio or video available later on, skip
             # all 'application/*' data types (fixes Linux Outlaws and peertube feeds)
-            if episode.mime_type.startswith('application/') and (audio_available or video_available):
+            if episode.mime_type.startswith('application/') and media_available:
                 continue
 
             episode.url = util.normalize_feed_url(enclosure['url'])
@@ -327,7 +354,14 @@ class PodcastEpisode(PodcastModelObject):
         if file_type is not None:
             return episode
 
-        return None
+        if link_has_media:
+            return episode
+
+        # The episode has no downloadable content.
+        # It is either a blog post or it links to a webpage with content accessible from shownotes title.
+        # Remove the URL so downloading will fail.
+        episode.url = ''
+        return episode
 
     def __init__(self, channel):
         self.parent = channel
@@ -340,8 +374,10 @@ class PodcastEpisode(PodcastModelObject):
         self.file_size = 0
         self.mime_type = 'application/octet-stream'
         self.guid = ''
+        self.episode_art_url = None
         self.description = ''
         self.description_html = ''
+        self.chapters = None
         self.link = ''
         self.published = 0
         self.download_filename = None
@@ -359,6 +395,9 @@ class PodcastEpisode(PodcastModelObject):
         # Timestamp of last playback time
         self.last_playback = 0
 
+        self._download_error = None
+        self._text_description = ''
+
     @property
     def channel(self):
         return self.parent
@@ -425,7 +464,68 @@ class PodcastEpisode(PodcastModelObject):
         if task is None:
             return False
 
-        return task.status in (task.DOWNLOADING, task.QUEUED, task.PAUSED)
+        return task.status in (task.DOWNLOADING, task.QUEUED, task.PAUSING, task.PAUSED, task.CANCELLING)
+
+    def get_player(self, config):
+        file_type = self.file_type()
+        if file_type == 'video' and config.player.video and config.player.video != 'default':
+            player = config.player.video
+        elif file_type == 'audio' and config.player.audio and config.player.audio != 'default':
+            player = config.player.audio
+        else:
+            player = 'default'
+        return player
+
+    def can_play(self, config):
+        """
+        # gPodder.playback_episodes() filters selection with this method.
+        """
+        return self.was_downloaded(and_exists=True) or self.can_stream(config)
+
+    def can_stream(self, config):
+        """
+        Don't try streaming if the user has not defined a player
+        or else we would probably open the browser when giving a URL to xdg-open.
+        We look at the audio or video player depending on its file type.
+        """
+        player = self.get_player(config)
+        return player and player != 'default'
+
+    def can_download(self):
+        """
+        gPodder.on_download_selected_episodes() filters selection with this method.
+        PAUSING and PAUSED tasks can be resumed.
+        """
+        return not self.was_downloaded(and_exists=True) and (
+            not self.download_task
+            or self.download_task.can_queue()
+            or self.download_task.status == self.download_task.PAUSING)
+
+    def can_pause(self):
+        """
+        gPodder.on_pause_selected_episodes() filters selection with this method.
+        """
+        return self.download_task and self.download_task.can_pause()
+
+    def can_cancel(self):
+        """
+        DownloadTask.cancel() only cancels the following tasks.
+        """
+        return self.download_task and self.download_task.can_cancel()
+
+    def can_delete(self):
+        """
+        gPodder.delete_episode_list() filters out locked episodes, and cancels all unlocked tasks in selection.
+        """
+        return self.state != gpodder.STATE_DELETED and not self.archive and (
+            not self.download_task or self.download_task.status == self.download_task.FAILED)
+
+    def can_lock(self):
+        """
+        gPodder.on_item_toggle_lock_activate() unlocks deleted episodes and toggles all others.
+        Locked episodes can always be unlocked.
+        """
+        return self.state != gpodder.STATE_DELETED or self.archive
 
     def check_is_new(self):
         return (self.state == gpodder.STATE_NORMAL and self.is_new and
@@ -475,9 +575,21 @@ class PodcastEpisode(PodcastModelObject):
 
     age_prop = property(fget=get_age_string)
 
+    def cache_text_description(self):
+        if self.description:
+            self._text_description = self.description
+        elif self.description_html:
+            self._text_description = util.remove_html_tags(self.description_html)
+        else:
+            self._text_description = ''
+
+    def html_description(self):
+        return self.description_html \
+            or util.nice_html_description(self.episode_art_url, self.description or _('No description available'))
+
     def one_line_description(self):
         MAX_LINE_LENGTH = 120
-        desc = util.remove_html_tags(self.description or '')
+        desc = self._text_description
         desc = re.sub(r'\s+', ' ', desc).strip()
         if not desc:
             return _('No description available')
@@ -496,6 +608,7 @@ class PodcastEpisode(PodcastModelObject):
             gpodder.user_extensions.on_episode_delete(self, filename)
             util.delete_file(filename)
 
+        self._download_error = None
         self.set_state(gpodder.STATE_DELETED)
 
     def get_playback_url(self, config=None, allow_partial=False):
@@ -568,7 +681,7 @@ class PodcastEpisode(PodcastModelObject):
         if not check_only and (force_update or not self.download_filename):
             # Avoid and catch gPodder bug 1440 and similar situations
             if template == '':
-                logger.warn('Empty template. Report this podcast URL %s',
+                logger.warning('Empty template. Report this podcast URL %s',
                         self.channel.url)
                 template = None
 
@@ -581,7 +694,7 @@ class PodcastEpisode(PodcastModelObject):
 
             if 'redirect' in episode_filename and template is None:
                 # This looks like a redirection URL - force URL resolving!
-                logger.warn('Looks like a redirection to me: %s', self.url)
+                logger.warning('Looks like a redirection to me: %s', self.url)
                 url = util.get_real_url(self.channel.authenticate_url(self.url))
                 logger.info('Redirection resolved to: %s', url)
                 episode_filename, _ = util.filename_from_url(url)
@@ -624,7 +737,7 @@ class PodcastEpisode(PodcastModelObject):
                     # call it from the downloading code before saving the file
                     logger.info('Choosing new filename: %s', new_file_name)
                 else:
-                    logger.warn('%s exists or %s does not', new_file_name, old_file_name)
+                    logger.warning('%s exists or %s does not', new_file_name, old_file_name)
                 logger.info('Updating filename of %s to "%s".', self.url, wanted_filename)
             elif self.download_filename is None:
                 logger.info('Setting download filename: %s', wanted_filename)
@@ -695,7 +808,7 @@ class PodcastEpisode(PodcastModelObject):
         try:
             return datetime.datetime.fromtimestamp(self.published).strftime('%H%M')
         except:
-            logger.warn('Cannot format pubtime: %s', self.title, exc_info=True)
+            logger.warning('Cannot format pubtime: %s', self.title, exc_info=True)
             return '0000'
 
     def playlist_title(self):
@@ -766,7 +879,8 @@ class PodcastEpisode(PodcastModelObject):
             return '-'
 
     def update_from(self, episode):
-        for k in ('title', 'url', 'description', 'description_html', 'link', 'published', 'guid', 'payment_url'):
+        for k in ('title', 'url', 'episode_art_url', 'description', 'description_html', 'chapters', 'link',
+                  'published', 'guid', 'payment_url'):
             setattr(self, k, getattr(episode, k))
         # Don't overwrite file size on downloaded episodes
         # See #648 refreshing a youtube podcast clears downloaded file size
@@ -850,7 +964,7 @@ class PodcastChannel(PodcastModelObject):
             logger.debug('Strategy for %s changed to %s', self.title, caption)
             self.download_strategy = download_strategy
         else:
-            logger.warn('Cannot set strategy to %d', download_strategy)
+            logger.warning('Cannot set strategy to %d', download_strategy)
 
     def rewrite_url(self, new_url):
         new_url = util.normalize_feed_url(new_url)
@@ -944,7 +1058,7 @@ class PodcastChannel(PodcastModelObject):
                         break
 
             if not found and not util.is_system_file(filename):
-                logger.warn('Unknown external file: %s', filename)
+                logger.warning('Unknown external file: %s', filename)
 
     @classmethod
     def sort_key(cls, podcast):
@@ -1003,7 +1117,9 @@ class PodcastChannel(PodcastModelObject):
 
         Returns: A new PodcastEpisode object
         """
-        return self.EpisodeClass.create_from_dict(d, self)
+        episode = self.EpisodeClass.create_from_dict(d, self)
+        episode.cache_text_description()
+        return episode
 
     def _consume_updated_title(self, new_title):
         # Replace multi-space and newlines with single space (Maemo bug 11173)
@@ -1243,7 +1359,9 @@ class PodcastChannel(PodcastModelObject):
         if self.title == new_title:
             return
 
-        new_folder_name = self.find_unique_folder_name(new_title)
+        fn_template = util.sanitize_filename(new_title, self.MAX_FOLDERNAME_LENGTH)
+
+        new_folder_name = self.find_unique_folder_name(fn_template)
         if new_folder_name and new_folder_name != self.download_folder:
             new_folder = os.path.join(gpodder.downloads, new_folder_name)
             old_folder = os.path.join(gpodder.downloads, self.download_folder)
@@ -1297,7 +1415,6 @@ class PodcastChannel(PodcastModelObject):
 
     def get_save_dir(self, force_new=False):
         if self.download_folder is None or force_new:
-            # we must change the folder name, because it has not been set manually
             fn_template = util.sanitize_filename(self.title, self.MAX_FOLDERNAME_LENGTH)
 
             if not fn_template:
