@@ -32,13 +32,14 @@ import os
 import sys
 import time
 
-import gpodder
 # Append gPodder's user agent to mygpoclient's user agent
 import mygpoclient
-from gpodder import minidb, util
 from mygpoclient import api, public
 from mygpoclient import util as mygpoutil
 
+import gpodder
+from gpodder import minidb, util
+
 _ = gpodder.gettext
 
 
@@ -286,7 +287,7 @@ class MygPoClient(object):
 
         The list might be empty. All these actions have to
         be processed. The user should confirm which of these
-        actions should be taken, the reest should be rejected.
+        actions should be taken, the rest should be rejected.
 
         Use confirm_received_actions and reject_received_actions
         to return and finalize the actions received by this
@@ -444,7 +445,7 @@ class MygPoClient(object):
 
     def flush(self, now=False):
         if not self.can_access_webservice():
-            logger.warn('Flush requested, but sync disabled.')
+            logger.warning('Flush requested, but sync disabled.')
             return
 
         if self._worker_thread is None or now:
@@ -511,7 +512,7 @@ class MygPoClient(object):
                 raise
 
             except Exception as e:
-                logger.warn('Exception while polling for episodes.', exc_info=True)
+                logger.warning('Exception while polling for episodes.', exc_info=True)
 
             # Step 2: Upload Episode actions
 
@@ -533,7 +534,7 @@ class MygPoClient(object):
             return True
 
         except (MissingCredentials, mygpoclient.http.Unauthorized):
-            logger.warn('Invalid credentials. Disabling gpodder.net.')
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
             self._config.mygpo.enabled = False
             return False
 
@@ -598,7 +599,7 @@ class MygPoClient(object):
             return True
 
         except (MissingCredentials, mygpoclient.http.Unauthorized):
-            logger.warn('Invalid credentials. Disabling gpodder.net.')
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
             self._config.mygpo.enabled = False
             return False
 
@@ -615,7 +616,7 @@ class MygPoClient(object):
             return True
 
         except (MissingCredentials, mygpoclient.http.Unauthorized):
-            logger.warn('Invalid credentials. Disabling gpodder.net.')
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
             self._config.mygpo.enabled = False
             return False
 
@@ -631,7 +632,7 @@ class MygPoClient(object):
             devices = self._client.get_devices()
 
         except (MissingCredentials, mygpoclient.http.Unauthorized):
-            logger.warn('Invalid credentials. Disabling gpodder.net.')
+            logger.warning('Invalid credentials. Disabling gpodder.net.')
             self._config.mygpo.enabled = False
             raise
 
