@@ -73,14 +73,14 @@ logger = logging.getLogger(__name__)
 try:
     import html5lib
 except ImportError:
-    logger.warn('html5lib not found, falling back to HTMLParser')
+    logger.warning("html5lib was not found, fall-back to HTMLParser")
     html5lib = None
 
 if gpodder.ui.win32:
     try:
         import gpodder.utilwin32ctypes as win32file
     except ImportError:
-        logger.warn('Running on Win32 but utilwin32ctypes can\'t be loaded.')
+        logger.warning('Running on Win32: utilwin32ctypes cannot be loaded')
         win32file = None
 
 _ = gpodder.gettext
@@ -90,7 +90,7 @@ N_ = gpodder.ngettext
 try:
     locale.setlocale(locale.LC_ALL, '')
 except Exception as e:
-    logger.warn('Cannot set locale (%s)', e, exc_info=True)
+    logger.warning('Cannot set locale (%s)', e, exc_info=True)
 
 # Native filesystem encoding detection
 encoding = sys.getfilesystemencoding()
@@ -148,26 +148,58 @@ _MIME_TYPE_LIST = [
     ('.wmv', 'video/x-ms-wmv'),
     ('.opus', 'audio/opus'),
     ('.webm', 'video/webm'),
+    ('.webm', 'audio/webm'),
 ]
 
 _MIME_TYPES = dict((k, v) for v, k in _MIME_TYPE_LIST)
 _MIME_TYPES_EXT = dict(_MIME_TYPE_LIST)
 
 
+def is_absolute_url(url):
+    """
+    Check if url is an absolute url (i.e. has a scheme)
+    """
+    try:
+        parsed = urllib.parse.urlparse(url)
+        # fix #1190: when parsing a windows path, scheme=drive_letter, path=\rest_of_path
+        return parsed.scheme and not parsed.path.startswith("\\")
+    except ValueError:
+        return False
+
+
+def new_gio_file(path):
+    """
+    Create a new Gio.File given a path or uri
+    """
+    from gi.repository import Gio
+
+    if is_absolute_url(path):
+        return Gio.File.new_for_uri(path)
+    else:
+        return Gio.File.new_for_path(path)
+
+
 def make_directory(path):
     """
     Tries to create a directory if it does not exist already.
     Returns True if the directory exists after the function
     call, False otherwise.
     """
-    if os.path.isdir(path):
+    from gi.repository import Gio, GLib
+
+    if not isinstance(path, Gio.File):
+        path = new_gio_file(path)
+
+    if path.query_exists():
         return True
 
     try:
-        os.makedirs(path)
-    except:
-        logger.warn('Could not create directory: %s', path)
-        return False
+        path.make_directory_with_parents()
+    except GLib.Error as err:
+        # The sync might be multithreaded, so directories can be created by other threads
+        if not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.EXISTS):
+            logger.warning('Could not create directory %s: %s', path.get_uri(), err.message)
+            return False
 
     return True
 
@@ -216,6 +248,12 @@ def normalize_feed_url(url):
     if not url or len(url) < 8:
         return None
 
+    # Removes leading and/or trailing whitespaces - if url contains whitespaces
+    # in between after str.strip() -> conclude invalid url & return None
+    url = url.strip()
+    if ' ' in url:
+        return None
+
     # This is a list of prefixes that you can use to minimize the amount of
     # keystrokes that you have to use.
     # Feel free to suggest other useful prefixes, and I'll add them here.
@@ -366,9 +404,9 @@ def calculate_size(path):
                 try:
                     sum += calculate_size(os.path.join(path, item))
                 except:
-                    logger.warn('Cannot get size for %s', path, exc_info=True)
+                    logger.warning('Cannot get size for %s', path, exc_info=True)
         except:
-            logger.warn('Cannot access %s', path, exc_info=True)
+            logger.warning('Cannot access %s', path, exc_info=True)
 
         return sum
 
@@ -392,7 +430,7 @@ def file_modification_datetime(filename):
         timestamp = s[stat.ST_MTIME]
         return datetime.datetime.fromtimestamp(timestamp)
     except:
-        logger.warn('Cannot get mtime for %s', filename, exc_info=True)
+        logger.warning('Cannot get mtime for %s', filename, exc_info=True)
         return None
 
 
@@ -419,7 +457,7 @@ def file_modification_timestamp(filename):
         s = os.stat(filename)
         return s[stat.ST_MTIME]
     except:
-        logger.warn('Cannot get modification timestamp for %s', filename)
+        logger.warning('Cannot get modification timestamp for %s', filename)
         return -1
 
 
@@ -510,10 +548,10 @@ def format_date(timestamp):
     try:
         timestamp_date = time.localtime(timestamp)[:3]
     except ValueError as ve:
-        logger.warn('Cannot convert timestamp', exc_info=True)
+        logger.warning('Cannot convert timestamp', exc_info=True)
         return None
     except TypeError as te:
-        logger.warn('Cannot convert timestamp', exc_info=True)
+        logger.warning('Cannot convert timestamp', exc_info=True)
         return None
 
     if timestamp_date == today:
@@ -524,7 +562,7 @@ def format_date(timestamp):
     try:
         diff = int((time.time() - timestamp) / seconds_in_a_day)
     except:
-        logger.warn('Cannot convert "%s" to date.', timestamp, exc_info=True)
+        logger.warning('Cannot convert "%s" to date.', timestamp, exc_info=True)
         return None
 
     try:
@@ -744,6 +782,10 @@ class ExtractHyperlinkedText(object):
         return self.extracter.get_result()
 
     def visit(self, element):
+        # skip functions generated by html5lib for comments in the HTML
+        if callable(element.tag):
+            return
+
         NS = '{http://www.w3.org/1999/xhtml}'
         tag_name = (element.tag[len(NS):] if element.tag.startswith(NS) else element.tag).lower()
         self.extracter.handle_starttag(tag_name, list(element.items()))
@@ -802,6 +844,24 @@ def extract_hyperlinked_text(html):
         return ExtractHyperlinkedTextHTMLParser()(html)
 
 
+def nice_html_description(img, description):
+    """
+    basic html formating + hyperlink highlighting + video thumbnail
+    """
+    description = re.sub(r'''https?://[^\s]+''',
+                         r'''<a href="\g<0>">\g<0></a>''',
+                         description)
+    description = description.replace('\n', '<br>')
+    html = """<style type="text/css">
+    body > img { float: left; max-width: 30vw; margin: 0 1em 1em 0; }
+    </style>
+    """
+    if img:
+        html += '<img src="{}">'.format(img)
+    html += '<p>{}</p>'.format(description)
+    return html
+
+
 def wrong_extension(extension):
     """
     Determine if a given extension looks like it's
@@ -1051,7 +1111,7 @@ def object_string_formatter(s, **kwargs):
                     to_s = str(getattr(o, attr))
                     result = result.replace(from_s, to_s)
                 except:
-                    logger.warn('Replace of "%s" failed for "%s".', attr, s)
+                    logger.warning('Replace of "%s" failed for "%s".', attr, s)
 
     return result
 
@@ -1416,7 +1476,7 @@ def http_request(url, method='HEAD'):
     return conn.getresponse()
 
 
-def gui_open(filename):
+def gui_open(filename, gui=None):
     """
     Open a file or folder with the default application set
     by the Desktop environment. This uses "xdg-open" on all
@@ -1427,13 +1487,30 @@ def gui_open(filename):
     try:
         if gpodder.ui.win32:
             os.startfile(filename)
+            opener = None
         elif gpodder.ui.osx:
-            Popen(['open', filename], close_fds=True)
+            opener = 'open'
         else:
-            Popen(['gio', 'open', filename], close_fds=True)
+            opener = 'xdg-open'
+
+        if opener:
+            opener_fullpath = shutil.which(opener)
+            if opener_fullpath is None:
+                raise Exception((_("System default program '%(opener)s' not found"))
+                    % {'opener': opener}
+                )
+            Popen([opener_fullpath, filename], close_fds=True)
         return True
     except:
         logger.error('Cannot open file/folder: "%s"', filename, exc_info=True)
+        if gui is not None:
+            if opener is None:
+                message = _("Cannot open file/folder '%(filename)s' using default program") % {'filename': filename}
+            else:
+                message = _("Cannot open '%(filename)s' using '%(opener)s'") \
+                                        % {'filename': filename, 'opener': opener}
+            gui.show_message_details(_('Cannot open file/folder'),
+                    str(sys.exc_info()[1]), message)
         return False
 
 
@@ -1444,6 +1521,21 @@ def open_website(url):
     make sure your system is set up correctly.
     """
     run_in_background(lambda: webbrowser.open(url))
+    return True
+
+
+def copy_text_to_clipboard(text):
+    """
+    Copies the specified text to both clipboards.
+    """
+    import gi
+    gi.require_version('Gtk', '3.0')
+    from gi.repository import Gdk, Gtk
+
+    clipboard = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
+    clipboard.set_text(text, -1)
+    clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+    clipboard.set_text(text, -1)
 
 
 def convert_bytes(d):
@@ -1622,34 +1714,17 @@ def isabs(string):
     return os.path.isabs(string)
 
 
-def commonpath(l1, l2, common=[]):
-    """
-    helper functions for relpath
-    Source: http://code.activestate.com/recipes/208993/
-    """
-    if len(l1) < 1: return (common, l1, l2)
-    if len(l2) < 1: return (common, l1, l2)
-    if l1[0] != l2[0]: return (common, l1, l2)
-    return commonpath(l1[1:], l2[1:], common + [l1[0]])
-
-
 def relpath(p1, p2):
     """
-    Finds relative path from p1 to p2
-    Source: http://code.activestate.com/recipes/208993/
+    Finds relative path from p2 to p1, like os.path.relpath but handles
+    uris. Returns None if no such path exists due to the paths being on
+    different devices.
     """
-    def pathsplit(s):
-        return s.split(os.path.sep)
-
-    (common, l1, l2) = commonpath(pathsplit(p1), pathsplit(p2))
-    p = []
-    if len(l1) > 0:
-        p = [('..' + os.sep) * len(l1)]
-    p = p + l2
-    if len(p) == 0:
-        return "."
-
-    return os.path.join(*p)
+    u1 = urllib.parse.urlparse(p1)
+    u2 = urllib.parse.urlparse(p2)
+    if u1.scheme and u2.scheme and (u1.scheme != u2.scheme or u1.netloc != u2.netloc):
+        return None
+    return os.path.relpath(u1.path, u2.path)
 
 
 def get_hostname():
@@ -1808,7 +1883,11 @@ def get_update_info():
     days_since_release = (datetime.datetime.today() - release_parsed).days
 
     def convert(s):
-        return tuple(int(x) for x in s.split('.'))
+        # Use both public and local version label, see PEP 440
+        pubv, locv = next(
+            (v[0], v[1] if len(v) > 1 else '') for v in (s.split('+'),))
+        return tuple(int(x) if x.isdigit() else x.lower()
+            for x in pubv.split('.') + (locv.split('.') if locv else []))
 
     up_to_date = (convert(gpodder.__version__) >= convert(latest_version))
 
@@ -1899,7 +1978,7 @@ def connection_available():
             return online
 
     except Exception as e:
-        logger.warn('Cannot get connection status: %s', e, exc_info=True)
+        logger.warning('Cannot get connection status: %s', e, exc_info=True)
         # When we can't determine the connection status, act as if we're online (bug 1730)
         return True
 
@@ -2212,3 +2291,61 @@ def response_text(response, default_encoding='utf-8'):
         return response.text
     else:
         return response.content.decode(default_encoding)
+
+
+def mount_volume_for_file(file, op=None):
+    """
+    Utility method to mount the enclosing volume for the given file in a blocking
+    fashion
+    """
+    import gi
+    gi.require_version('Gio', '2.0')
+    from gi.repository import Gio, GLib
+    if gpodder.ui.gtk:
+        gi.require_version('Gtk', '3.0')
+        from gi.repository import Gtk
+    else:
+        loop = GLib.MainLoop()
+
+    result = True
+    message = None
+
+    def callback(file, res):
+        nonlocal result, message
+        try:
+            file.mount_enclosing_volume_finish(res)
+            result = True
+        except GLib.Error as err:
+            if (not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.NOT_SUPPORTED) and
+                    not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.ALREADY_MOUNTED)):
+                message = err.message
+                result = False
+        finally:
+            if gpodder.ui.gtk:
+                Gtk.main_quit()
+            else:
+                loop.quit()
+
+    file.mount_enclosing_volume(Gio.MountMountFlags.NONE, op, None, callback)
+    if gpodder.ui.gtk:
+        Gtk.main()
+    else:
+        loop.run()
+    return result, message
+
+
+def scale_pixbuf(pixbuf, max):
+    import gi
+    from gi.repository import GdkPixbuf
+
+    w_cur = pixbuf.get_width()
+    h_cur = pixbuf.get_height()
+
+    if w_cur <= max and h_cur <= max:
+        return pixbuf
+
+    f = max / (w_cur if w_cur >= h_cur else h_cur)
+    w_new = int(w_cur * f)
+    h_new = int(h_cur * f)
+
+    return pixbuf.scale_simple(w_new, h_new, GdkPixbuf.InterpType.BILINEAR)
