@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Manage Youtube subscriptions using youtube-dl (https://github.com/ytdl-org/youtube-dl)
+# Manage YouTube subscriptions using youtube-dl (https://github.com/ytdl-org/youtube-dl)
 # Requirements: youtube-dl module (pip install youtube_dl)
 # (c) 2019-08-17 Eric Le Lay <elelay.fr:contact>
 # Released under the same license terms as gPodder itself.
@@ -10,12 +10,17 @@ import re
 import sys
 import time
 
-import youtube_dl
-from youtube_dl.utils import DownloadError, ExtractorError, sanitize_url
+try:
+    import yt_dlp as youtube_dl
+except:
+    import youtube_dl
 
 import gpodder
-from gpodder import download, feedcore, model, registry, youtube
-from gpodder.util import mimetype_from_extension, remove_html_tags
+from gpodder import download, feedcore, model, registry, util, youtube
+
+import gi  # isort:skip
+gi.require_version('Gtk', '3.0')  # isort:skip
+from gi.repository import Gtk  # isort:skip
 
 _ = gpodder.gettext
 
@@ -23,14 +28,14 @@ _ = gpodder.gettext
 logger = logging.getLogger(__name__)
 
 
-__title__ = 'Youtube-dl'
-__description__ = _('Manage Youtube subscriptions using youtube-dl (pip install youtube_dl)')
+__title__ = 'youtube-dl'
+__description__ = _('Manage YouTube subscriptions using youtube-dl (pip install youtube_dl) or yt-dlp (pip install yt-dlp)')
 __only_for__ = 'gtk, cli'
 __authors__ = 'Eric Le Lay <elelay.fr:contact>'
 __doc__ = 'https://gpodder.github.io/docs/extensions/youtubedl.html'
 
-want_ytdl_version = '2020.11.12'
-want_ytdl_version_msg = _('Your version of youtube-dl %(have_version)s has known issues, please upgrade to %(want_version)s or newer.')
+want_ytdl_version = '2021.02.04'
+want_ytdl_version_msg = _('Your version of youtube-dl/yt-dlp %(have_version)s has known issues, please upgrade to %(want_version)s or newer.')
 
 DefaultConfig = {
     # youtube-dl downloads and parses each video page to get informations about it, which is very slow.
@@ -51,7 +56,7 @@ PLAYLIST_RE = re.compile(r'''https://www.youtube.com/feeds/videos.xml\?playlist_
 def youtube_parsedate(s):
     """Parse a string into a unix timestamp
 
-    Only strings provided by Youtube-dl API are
+    Only strings provided by youtube-dl API are
     parsed with this function (20170920).
     """
     if s:
@@ -85,14 +90,10 @@ class YoutubeCustomDownload(download.CustomDownload):
         """
         self._reporthook = reporthook
         # outtmpl: use given tempname by DownloadTask
-        # (escape % and $ because outtmpl used as a string template by youtube-dl)
-        outtmpl = tempname.replace('%', '%%').replace('$', '$$')
+        # (escape % because outtmpl used as a string template by youtube-dl)
+        outtmpl = tempname.replace('%', '%%')
         res = self._ytdl.fetch_video(self._url, outtmpl, self._my_hook)
-        if outtmpl != tempname:
-            if 'ext' in res and os.path.isfile(outtmpl + '.{}'.format(res['ext'])):
-                os.rename(outtmpl + '.{}'.format(res['ext']), tempname)
-            else:
-                os.rename(outtmpl, tempname)
+        # Renaming is not required because the escaped percent is not escaped in the output file.
         if 'duration' in res and res['duration']:
             self._episode.total_time = res['duration']
         headers = {}
@@ -100,22 +101,24 @@ class YoutubeCustomDownload(download.CustomDownload):
         if 'ext' in res:
             dot_ext = '.{}'.format(res['ext'])
             # See #673 when merging multiple formats, the extension is appended to the tempname
-            # by YoutubeDL resulting in empty .partial file + .partial.mp4 exists
+            # by youtube-dl resulting in empty .partial file + .partial.mp4 exists
             # and #796 .mkv is chosen by ytdl sometimes
-            tempstat = os.stat(tempname)
-            if not tempstat.st_size:
-                for try_ext in (dot_ext, ".mp4", ".m4a", ".webm", ".mkv"):
-                    tempname_with_ext = tempname + try_ext
-                    if os.path.isfile(tempname_with_ext):
-                        logger.debug('Youtubedl downloaded to "%s" instead of "%s", moving',
-                                     os.path.basename(tempname_with_ext),
-                                     os.path.basename(tempname))
-                        os.remove(tempname)
-                        os.rename(tempname_with_ext, tempname)
-                        dot_ext = try_ext
-                        break
-            ext_filetype = mimetype_from_extension(dot_ext)
+            for try_ext in (dot_ext, ".mp4", ".m4a", ".webm", ".mkv"):
+                tempname_with_ext = tempname + try_ext
+                if os.path.isfile(tempname_with_ext):
+                    logger.debug('youtube-dl downloaded to "%s" instead of "%s", moving',
+                                 os.path.basename(tempname_with_ext),
+                                 os.path.basename(tempname))
+                    os.remove(tempname)
+                    os.rename(tempname_with_ext, tempname)
+                    dot_ext = try_ext
+                    break
+            ext_filetype = util.mimetype_from_extension(dot_ext)
             if ext_filetype:
+                # YouTube weba formats have a webm extension and get a video/webm mime-type
+                # but audio content has no width or height, so change it to audio/webm for correct icon and player
+                if ext_filetype.startswith('video/') and ('height' not in res or res['height'] is None):
+                    ext_filetype = ext_filetype.replace('video/', 'audio/')
                 headers['content-type'] = ext_filetype
         return headers, res.get('url', self._url)
 
@@ -172,7 +175,7 @@ class YoutubeFeed(model.Feed):
         return filtered_entries
 
     def get_title(self):
-        return '{} (Youtube)'.format(self._ie_result.get('title') or self._ie_result.get('id') or self._url)
+        return '{} (YouTube)'.format(self._ie_result.get('title') or self._ie_result.get('id') or self._url)
 
     def get_link(self):
         return self._ie_result.get('webpage_url')
@@ -208,10 +211,8 @@ class YoutubeFeed(model.Feed):
         episodes = []
         for en in self._ie_result['entries']:
             guid = video_guid(en['id'])
-            description = remove_html_tags(en.get('description') or _('No description available'))
-            html_description = self.nice_html_description(en, description)
             if en.get('ext'):
-                mime_type = mimetype_from_extension('.{}'.format(en['ext']))
+                mime_type = util.mimetype_from_extension('.{}'.format(en['ext']))
             else:
                 mime_type = 'application/octet-stream'
             if en.get('filesize'):
@@ -222,8 +223,9 @@ class YoutubeFeed(model.Feed):
             ep = {
                 'title': en.get('title', guid),
                 'link': en.get('webpage_url'),
-                'description': description,
-                'description_html': html_description,
+                'episode_art_url': en.get('thumbnail'),
+                'description': util.remove_html_tags(en.get('description') or ''),
+                'description_html': '',
                 'url': en.get('webpage_url'),
                 'file_size': filesize,
                 'mime_type': mime_type,
@@ -246,25 +248,6 @@ class YoutubeFeed(model.Feed):
         """
         return None
 
-    @staticmethod
-    def nice_html_description(en, description):
-        """
-        basic html formating + hyperlink highlighting + video thumbnail
-        """
-        description = re.sub(r'''https?://[^\s]+''',
-                             r'''<a href="\g<0>">\g<0></a>''',
-                             description)
-        description = description.replace('\n', '<br>')
-        html = """<style type="text/css">
-        body > img { float: left; max-width: 30vw; margin: 0 1em 1em 0; }
-        </style>
-        """
-        img = en.get('thumbnail')
-        if img:
-            html += '<img src="{}">'.format(img)
-        html += '<p>{}</p>'.format(description)
-        return html
-
 
 class gPodderYoutubeDL(download.CustomDownloader):
     def __init__(self, gpodder_config, my_config, force=False):
@@ -280,16 +263,21 @@ class gPodderYoutubeDL(download.CustomDownloader):
         self._ydl_opts = {
             'cachedir': cachedir,
             'no_color': True,  # prevent escape codes in desktop notifications on errors
+            'noprogress': True,  # prevent progress bar from appearing in console
         }
         if gpodder.verbose:
             self._ydl_opts['verbose'] = True
         else:
             self._ydl_opts['quiet'] = True
+        # Don't create downloaders for URLs supported by these youtube-dl extractors
+        self.ie_blacklist = ["Generic"]
+        # Cache URL regexes from youtube-dl matches here, seed with youtube regex
+        self.regex_cache = [re.compile(r'https://www.youtube.com/watch\?v=.+')]
         # #686 on windows without a console, sys.stdout is None, causing exceptions
         # when adding podcasts.
         # See https://docs.python.org/3/library/sys.html#sys.__stderr__ Note
         if not sys.stdout:
-            logger.debug('no stdout, setting YoutubeDL logger')
+            logger.debug('no stdout, setting youtube-dl logger')
             self._ydl_opts['logger'] = logger
 
     def add_format(self, gpodder_config, opts, fallback=None):
@@ -338,8 +326,8 @@ class gPodderYoutubeDL(download.CustomDownloader):
                 with youtube_dl.YoutubeDL(opts) as ydl:
                     ydl.process_ie_result(tmp, download=False)
                     new_entries.extend(tmp.get('entries'))
-            except DownloadError as ex:
-                if ex.exc_info[0] == ExtractorError:
+            except youtube_dl.utils.DownloadError as ex:
+                if ex.exc_info[0] == youtube_dl.utils.ExtractorError:
                     # for instance "This video contains content from xyz, who has blocked it on copyright grounds"
                     logger.warning('Skipping %s: %s', e.get('title', ''), ex.exc_info[1])
                     continue
@@ -372,7 +360,7 @@ class gPodderYoutubeDL(download.CustomDownloader):
             result_type, has_playlist = extract_type(ie_result)
             while not has_playlist:
                 if result_type in ('url', 'url_transparent'):
-                    ie_result['url'] = sanitize_url(ie_result['url'])
+                    ie_result['url'] = youtube_dl.utils.sanitize_url(ie_result['url'])
                 if result_type == 'url':
                     logger.debug("extract_info(%s) to get the video list", ie_result['url'])
                     # We have to add extra_info to the results because it may be
@@ -407,20 +395,40 @@ class gPodderYoutubeDL(download.CustomDownloader):
                 if m:
                     url = 'https://www.youtube.com/playlist?list={}'.format(m.group(1))
         if url:
-            logger.info('Youtube-dl Handling %s => %s', channel.url, url)
+            logger.info('youtube-dl handling %s => %s', channel.url, url)
             return self.refresh(url, channel.url, max_episodes)
         return None
 
+    def is_supported_url(self, url):
+        if url is None:
+            return False
+        if self.regex_cache[0].match(url) is not None:
+            return True
+        for r in self.regex_cache[1:]:
+            if r.match(url) is not None:
+                self.regex_cache.remove(r)
+                self.regex_cache.insert(0, r)
+                return True
+        with youtube_dl.YoutubeDL(self._ydl_opts) as ydl:
+            # youtube-dl returns a list, yt-dlp returns a dict
+            ies = ydl._ies
+            if type(ydl._ies) == dict:
+                ies = ydl._ies.values()
+            for ie in ies:
+                if ie.suitable(url) and ie.ie_key() not in self.ie_blacklist:
+                    self.regex_cache.insert(0, ie._VALID_URL_RE)
+                    return True
+        return False
+
     def custom_downloader(self, unused_config, episode):
         """
         called from registry.custom_downloader.resolve
         """
         if not self.force and not self.my_config.manage_downloads:
             return None
-        if re.match(r'''https://www.youtube.com/watch\?v=.+''', episode.url):
+        if self.is_supported_url(episode.url):
             return YoutubeCustomDownload(self, episode.url, episode)
-        elif re.match(r'''https://www.youtube.com/watch\?v=.+''', episode.link):
-            return YoutubeCustomDownload(self, episode.link, episode)
+
         return None
 
 
@@ -435,7 +443,7 @@ class gPodderExtension:
         registry.feed_handler.register(self.ytdl.fetch_channel)
         registry.custom_downloader.register(self.ytdl.custom_downloader)
 
-        logger.debug('Youtube-DL %s' % youtube_dl.version.__version__)
+        logger.debug('youtube-dl %s' % youtube_dl.version.__version__)
 
         if youtube_dl.utils.version_tuple(youtube_dl.version.__version__) < youtube_dl.utils.version_tuple(want_ytdl_version):
             logger.error(want_ytdl_version_msg
@@ -460,14 +468,52 @@ class gPodderExtension:
             if youtube_dl.utils.version_tuple(youtube_dl.version.__version__) < youtube_dl.utils.version_tuple(want_ytdl_version):
                 ui_object.notification(want_ytdl_version_msg %
                     {'have_version': youtube_dl.version.__version__, 'want_version': want_ytdl_version},
-                    _('Old Youtube-DL'), important=True, widget=ui_object.main_window)
+                    _('Old youtube-dl'), important=True, widget=ui_object.main_window)
 
     def on_episodes_context_menu(self, episodes):
-        if not self.container.config.manage_downloads \
-                and not all(e.was_downloaded(and_exists=True) for e in episodes):
-            return [(_("Download with Youtube-DL"), self.download_episodes)]
+        if not self.container.config.manage_downloads and any(e.can_download() for e in episodes):
+            return [(_("Download with youtube-dl"), self.download_episodes)]
 
     def download_episodes(self, episodes):
+        episodes = [e for e in episodes if e.can_download()]
+
         # create a new gPodderYoutubeDL to force using it even if manage_downloads is False
         downloader = gPodderYoutubeDL(self.container.manager.core.config, self.container.config, force=True)
         self.gpodder.download_episode_list(episodes, downloader=downloader)
+
+    def toggle_manage_channel(self, widget):
+        self.container.config.manage_channel = widget.get_active()
+
+    def toggle_manage_downloads(self, widget):
+        self.container.config.manage_downloads = widget.get_active()
+
+    def show_preferences(self):
+        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=10)
+        box.set_border_width(10)
+
+        checkbox = Gtk.CheckButton(_('Parse YouTube channel feeds with youtube-dl to access more than 15 episodes'))
+        checkbox.set_active(self.container.config.manage_channel)
+        checkbox.connect('toggled', self.toggle_manage_channel)
+        box.pack_start(checkbox, False, False, 0)
+
+        box.pack_start(Gtk.HSeparator(), False, False, 0)
+
+        checkbox = Gtk.CheckButton(_('Download all supported episodes with youtube-dl'))
+        checkbox.set_active(self.container.config.manage_downloads)
+        checkbox.connect('toggled', self.toggle_manage_downloads)
+        box.pack_start(checkbox, False, False, 0)
+        note = Gtk.Label(use_markup=True, wrap=True, label=_(
+            'youtube-dl provides access to additional YouTube formats and DRM content.'
+            '  Episodes from non-YouTube channels, that have youtube-dl support, will <b>fail</b> to download unless you manually'
+            ' <a href="https://gpodder.github.io/docs/youtube.html#formats">add custom formats</a> for each site.'
+            '  <b>Download with youtube-dl</b> appears in the episode menu when this option is disabled,'
+            ' and can be used to manually download from supported sites.'))
+        note.connect('activate-link', lambda label, url: util.open_website(url))
+        note.set_property('xalign', 0.0)
+        box.add(note)
+
+        box.show_all()
+        return box
+
+    def on_preferences(self):
+        return [(_('youtube-dl'), self.show_preferences)]
